/*
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2011 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 */

package org.ow2.orchestra.designer.server;


import java.util.Iterator;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import org.ow2.orchestra.designer.fileNavigator.Process;


/**
 * This class is a singleton designed to deal with tokens on opened processes.
 *
 * @author diouf-a
 */
public class ConcurrentProcessesManager {

  // Singleton pattern
  private static ConcurrentProcessesManager concurrentManager = null;

  // each entry contents the path process and the login of user which have opened process in writing
  private final Map<String, String> processesOpenedInWriting = new ConcurrentHashMap<String, String>();

  /**
   * Private constructor of class.
   * Initializes the map
   */
  private ConcurrentProcessesManager() {
  }

  /**
   * make one instance of manager of concurrent accesses, if not exist
   *
   * @return instance of manager
   */
  public static synchronized ConcurrentProcessesManager getInstance() {
    if (concurrentManager == null) {
      concurrentManager = new ConcurrentProcessesManager();
    }
    return concurrentManager;
  }

  /**
   * take writing access of process, if process isn't already opened in writing
   * and upgrade the map with the entry "pathProcess : user"
   *
   * @param process : the path of process
   * @param user    : the current user who want to open process
   */
  public void takeToken(Process process, String user) {
    final String key = this.getProcessKey(process);
    synchronized (processesOpenedInWriting) {
      if (!processesOpenedInWriting.containsKey(key)) {
        processesOpenedInWriting.put(key, user);
      }
    }
  }

  /**
   * free the token on process if "user" is the user who have the token
   *
   * @param prc  the given process
   * @param user the given user
   */
  public void freeToken(Process prc, String user) {
    final String key = this.getProcessKey(prc);
    synchronized (processesOpenedInWriting) {
      final String actualUser = processesOpenedInWriting.get(key);
      if (actualUser != null && actualUser.equals(user)) {
        processesOpenedInWriting.remove(key);
      }
    }
  }

  /**
   * return the user who have the token on the process "prc"
   *
   * @param prc the given process
   * @return The name of the user having the token on a process or null if there is no lock on this process
   */
  public String checkWhoTakeToken(Process prc) {
    String key = this.getProcessKey(prc);
    synchronized (processesOpenedInWriting) {
      return processesOpenedInWriting.get(key);
    }
  }


  /**
   * free all tokens on processes opened by a user
   *
   * @param user the given user
   */
  public void freeTokensObtainedByUser(String user) {
    synchronized (processesOpenedInWriting) {
      Iterator<Map.Entry<String, String>> iterator = processesOpenedInWriting.entrySet().iterator();
      while (iterator.hasNext()) {
        Map.Entry<String, String> e = iterator.next();
        if (e.getValue().equals(user)) {
          iterator.remove();
        }
      }
    }
  }

  /**
   * determine the entry key of a process
   *
   * @param process The process
   * @return return the key for the given process
   */
  private String getProcessKey(Process process) {
    return process.getProject().getOwner().getName() + "-" + process.getProject().getName() + "-" + process.getName();
  }
}
